import json
import logging

import requests
from orchestrator import inputstep
from orchestrator.config.assignee import Assignee
from orchestrator.forms import FormPage, ReadOnlyField
from orchestrator.forms.validators import Accept, Label, LongText
from orchestrator.types import UUIDstr, State
from orchestrator.utils.json import json_dumps

from gso import settings
from gso.products.product_types.device import DeviceProvisioning

logger = logging.getLogger(__name__)


def provision_node(
        node_subscription_params: DeviceProvisioning,
        process_id: UUIDstr,
        dry_run: bool = True):
    oss = settings.load_oss_params()
    pp_params = oss.PROVISIONING_PROXY
    assert pp_params

    device_params = node_subscription_params.device
    callback_url = f'{settings.load_oss_params().GENERAL.public_hostname}' \
                   f'/api/processes/{process_id}/resume'
    logger.debug(f'[disabled] provisioning node {device_params}')

    parameters = {
        'callback': callback_url,
        'dry_run': dry_run,
        'device': json.loads(json_dumps(node_subscription_params))
    }

    post_request = requests.post(
        f'{pp_params.scheme}://{pp_params.api_base}'
        f'/api/device',
        json=parameters)
    post_request.raise_for_status()


@inputstep('Await provisioning proxy results', assignee=Assignee('SYSTEM'))
def await_pp_results() -> State:
    class ProvisioningResultPage(FormPage):
        class Config:
            title = 'Do NOT click on confirm in this step!'

        warning_label: Label = 'This step relies on an external service to ' \
                               'send an update to the orchestrator, do not ' \
                               'interfere with this process please.'
        pp_run_results: dict = {'state': 'not_ready'}
        confirm: Accept = Accept('INCOMPLETE')

    result_page = yield ProvisioningResultPage

    return result_page.dict()


@inputstep('Confirm provisioning proxy results', assignee=Assignee('SYSTEM'))
def confirm_pp_results(state: State) -> State:
    class ConfirmRunPage(FormPage):
        class Config:
            title = 'Execution completed, please confirm the results.'

        run_status: str = ReadOnlyField(state['pp_run_results']['status'])
        run_results: LongText = ReadOnlyField(
            f"{state['pp_run_results']['output']}")
        confirm: Accept = Accept('INCOMPLETE')

    yield ConfirmRunPage

    return state
